/*  FUNNEL LIBRARY
**  
**  Array -- a library supporting arrays
**  
**  16/08/2K, Matthias Zenger
*/

newtype ArrayModule = {
    def create[a](n: int): Array[a]
    def createInitialized[a](n: int, x: a): Array[a]
    def make[a](x: a): Array[a]
    def make2[a](x: a, y: a): Array[a]
    def make3[a](x: a, y: a, z: a): Array[a]
    def make4[a](x: a, b: a, c: a, d: a): Array[a]
    def make5[a](x: a, b: a, c: a, d: a, e: a): Array[a]
    def make6[a](x: a, b: a, c: a, d: a, e: a, f: a): Array[a]
    def createJavaArray(t: null)(n: int): Array[any]
}

newtype Array[a] = {
    def apply(n: int): a
    def at(n: int): a
    def update(n: int)(x: a): ()
    def length: int
    def for(from: int)(to: int)(do: a -> ()): ()
    def iterate(do: a -> ()): ()
    def updateFor(from: int)(to: int)(do: int -> a): ()
    def updateAll(do: int -> a): ()
    def toList: List[a]
    def toString: string
    def toJavaArray: any
}

let Array: ArrayModule = {
    with Java
    
    def createArray[a](t: JavaClass)(length: int): Array[a] = {
        val toJavaArray = {
            jmethod "newInstance" (java_lang_Class, int_type)
                java_lang_reflect_Array t length
        }
        val at = {
            jmethod "get" (java_lang_Object, int_type)
                java_lang_reflect_Array toJavaArray
        }
        val update' = {
            jmethod "set" (java_lang_Object, int_type, java_lang_Object)
                java_lang_reflect_Array toJavaArray
        }
        def apply n = {
            if ((n >= length) || (n < 0))
                System.error("index " + n + " out of bounds")
            else
                at n
        }
        def update n x = {
            if ((n >= length) || (n < 0))
                System.error("index " + n + " out of bounds")
            else
                update' n x
        }
        def for from to do = {
            if (from <= to) {
                do (at from)
                for (from + 1) to do
            } else
                ()
        }
        val iterate = for 0 (length - 1)
        def updateFor from to do = {
            if (from <= to) {
                update from (do(from))
                updateFor (from + 1) to do
            } else
                ()
        }
        val updateAll = updateFor 0 (length - 1)
        def toList'(n: int): List[a] = {
            if (n < 0) List.Nil else List.Cons(at n, toList'(n - 1))
        }
        def toList = toList'(length - 1)
        def toString = {
            if (length == 0)
                "{}"
            else {
                var return: string := "{" + at(0)
                for 1 (length - 1) (x| return := return + ", " + x)
                return + "}"
            }
        }
    }
    def create[t] n = createArray[t] java_lang_Object n
    def createInitialized[t](n, x) = {
        val arr = createArray[t] java_lang_Object n
        arr.updateAll(i| x)
        arr
    }
    def make[t](x) = {
        val as = create[t](1)
        as(0) := x
        as
    }
    def make2[t](x, y) = {
        val as = create[t](2)
        as(0) := x; as(1) := y
        as
    }
    def make3[t](x, y, z) = {
        val as = create[t](3)
        as(0) := x; as(1) := y; as(2) := z
        as
    }
    def make4[t](a, b, c, d) = {
        val as = create[t](4)
        as(0) := a; as(1) := b; as(2) := c; as(3) := d
        as
    }
    def make5[t](a, b, c, d, e) = {
        val as = create[t](5)
        as(0) := a; as(1) := b; as(2) := c; as(3) := d; as(4) := e
        as
    }
    def make6[t](a, b, c, d, e, f) = {
        val as = create[t](6)
        as(0) := a; as(1) := b; as(2) := c; as(3) := d; as(4) := e; as(5) := f
        as
    }
    def fromList[t](xs: List[t]) = {
        val arr = create[t](xs.length)
        def fill(n: int, xs: List[t]): () =
            if (n == arr.length) ()
            else (arr(n) := xs.head; fill(n + 1, xs.tail))
        fill(0, xs)
        arr
    }
    val createJavaArray = createArray[JavaObject]
}
