/*  FUNNEL LIBRARY
**  
**  Console -- a library that interfaces with text consoles
**  
**  11/06/2K, Matthias Zenger
*/

type JavaObject = any

newtype ConsoleModule = {
    def in: JavaObject
    def out: JavaObject
    def instream: JavaObject
    def print(x: any): ()
    def println(x: any): ()
    def readln: string
    def readInt: int
    def readFloat: float
}

/** a synchronous implementation of console i/o; i.e. the
 *  input functions are blocking
 */
let Console: ConsoleModule = {
    val java_io_InputStream = Java.jclass "java.io.InputStream"
    val java_io_DataInputStream = Java.jclass "java.io.DataInputStream"
    val in = Java.jfield "in" (Java.java_lang_System)
    val out = Java.jfield "out" (Java.java_lang_System)
    val instream = Java.jmethod "<init>" (java_io_InputStream)
                       (java_io_DataInputStream) in

    val print = Java.jmethod "print" (Java.java_lang_Object) out
    val println = Java.jmethod "println" (Java.java_lang_Object) out

    def readln = Java.jmethod "readLine" () instream
    def readInt = String.toInt readln
    def readFloat = String.toFloat readln
}

/** asynchronous input functions; i.e. these functions are
 *  executed concurrently with the Funnel code
 */
let AsyncConsole: ConsoleModule = {
    with Console
    def readln = Java.ajmethod "readLine" () instream
    def readInt = String.toInt readln
    def readFloat = String.toFloat readln
}
