/*  FUNNEL LIBRARY
**  
**  Java -- java interface for Funnel; this interface is untyped
**          and accessing java methods and fields may result in
**          runtime errors
**  
**  11/06/2K, Matthias Zenger
*/

type JavaObject = any
type JavaClass = null
type ParameterList = any

newtype JavaModule = {
    def jclass: string -> JavaClass
    def jmethod: string -> ParameterList -> JavaObject -> null
    def jconstructor: ParameterList -> JavaClass -> null
    def jfield: string -> JavaObject -> null
    def ajmethod: string -> ParameterList -> JavaObject -> null
    def java_lang_Object: JavaClass
    def java_lang_reflect_Array: JavaClass
    def java_lang_Class: JavaClass
    def java_lang_System: JavaClass
    def java_lang_String: JavaClass
    def java_lang_Integer: JavaClass
    def java_lang_Long: JavaClass
    def java_lang_Float: JavaClass
    def java_lang_Double: JavaClass
    def java_lang_Character: JavaClass
    def byte_type: JavaClass
    def short_type: JavaClass
    def char_type: JavaClass
    def int_type: JavaClass
    def long_type: JavaClass
    def float_type: JavaClass
    def double_type: JavaClass
    def boolean_type: JavaClass
}

let Java: JavaModule = {
    val jclass: string -> JavaClass = primitive 1
    val jmethod: string -> ParameterList -> JavaObject -> null = primitive 2
    val jconstructor: ParameterList -> JavaClass -> null = primitive 2 "<init>"
    val jfield: string -> JavaObject -> null = primitive 3
    val ajmethod: string -> ParameterList -> JavaObject -> null = primitive 4

    // some common java classes

    val java_lang_Object = jclass "java.lang.Object"
    val java_lang_reflect_Array = jclass "java.lang.reflect.Array"
    val java_lang_Class = jclass "java.lang.Class"
    val java_lang_System = jclass "java.lang.System"
    val java_lang_String = jclass "java.lang.String"
    val java_lang_Integer = jclass "java.lang.Integer"
    val java_lang_Long = jclass "java.lang.Long"
    val java_lang_Float = jclass "java.lang.Float"
    val java_lang_Double = jclass "java.lang.Double"
    val java_lang_Character = jclass "java.lang.Character"

    // some basic java types
    val byte_type = jclass "byte"
    val short_type = jclass "short"
    val char_type = jclass "char"
    val int_type = jclass "int"
    val long_type = jclass "long"
    val float_type = jclass "float"
    val double_type = jclass "double"
    val boolean_type = jclass "boolean"
}
