/*  FUNNEL LIBRARY
**  
**  Predef -- some basic predefined functions for Funnel
**  
**  11/06/2K, Matthias Zenger
*/

newtype PredefModule = {
    def fst[a, b](x: a, y: b): a
    def snd[a, b](x: a, y: b): b
    def curry[a, b, c](f: (a, b) -> c)(x: a)(y: b): c
    def uncurry[a, b, c](f: a -> b -> c)(x: a, y: b): c
    def id[a](x: a): a
    def const[a](k: a)(l: any): a
    def compose[b, c](f: b -> c)[a](g: a -> b)(x: a): c
    def flip[a, b, c](f: a -> b -> c)(x: b)(y: a): c
    def until[a](p: a -> boolean)(f: a -> a)(x: a): a
    def abs(x: int): int
    def while(cond: () -> boolean)(body: () -> ()): ()
    def for(start: int)(stop: int)(by: int)(do: int -> ()): ()
}

let Predef: PredefModule = {

    // general functions
    def fst[a, b](x: a, y: b): a = x
    def snd[a, b](x: a, y: b): b = y
    def curry[a, b, c](f: (a, b) -> c)(x: a)(y: b): c = f(x, y)
    def uncurry[a, b, c](f: a -> b -> c)(x: a, y: b): c = f x y
    def id[a](x: a): a = x
    def const[a](k: a)(l: any): a = k
    def compose[b, c](f: b -> c)[a](g: a -> b)(x: a): c = f(g x)
    def flip[a, b, c](f: a -> b -> c)(x: b)(y: a): c = f y x
    def until[a](p: a -> boolean)(f: a -> a)(x: a): a =
                                            if (p x) x else until[a] p f (f x)

    def abs(x: int): int = if (x < 0) -x else x

    // control constructs

    def while(cond: () -> boolean)(body: () -> ()): () = {
        if (cond()) {
            body(); while cond body
        } else
            ()
    }

    def for(start: int)(stop: int)(by: int)(do: int -> ()): () = {
        if (start <= stop) {
            do start
            for (start + by) stop by do
        } else
            ()
    }
}
