/*  FUNNEL LIBRARY
**  
**  Stream -- support for streams in Funnel
**  
**  11/06/2K, Matthias Zenger
*/

newtype StreamModule = {
    def fromString(str: string): List[char]
    def fromFile(fname: string): List[char]
    def numbers(from: int): List[int]
    def append[a](xs: List[a])(ys: List[a]): List[a]
    def map[a, b](f: a -> b)(xs: List[a]): List[b]
    def zip[a, b](xs: List[a])(ys: List[b]): List[(a, b)]
    def zipWith[a, b, c](f: (a, b) -> c)(xs: List[a])(ys: List[b]): List[c]
    def filter[a](f: a -> boolean)(xs: List[a]): List[a]
}

let Stream: StreamModule = {

    def fromString str = {
        def from(str: string, xs: List[char], n: int): List[char] = {
            let res = if (n < 0) xs
                      else from(str, List.Cons(str(n), xs), n - 1)
            res
        }
        from(str, List.Nil[char], str.length - 1)
    }

    def fromFile fname = {
        val in = Java.jconstructor (Java.java_lang_String)
                              (Java.jclass "java.io.FileInputStream") fname
        val c = Java.jmethod "read" () in
        def read = {
            let res = if (c == -1) (Java.jmethod "close" () in; List.Nil)
                      else List.Cons(c.toChar, read)
            res
        }
        read
    }
    
    def numbers(from) = {
        let res = List.Cons(from, numbers(from + 1))
        res
    }
    
    def append[t] xs ys = {
        let res = xs.appendL ys
        res
    }
    
    def map[s, t] f xs = {
        let res = xs.mapL f
        res
    }
    
    def zip[s, t] xs ys = {
        let res = xs.zipL ys
        res
    }
    
    def zipWith[s, t, u] f xs ys = {
        let res = xs.zipWithL[t, u] f ys
        res
    }
    
    def filter[t] f xs = {
        let res = xs.filterL(f)
        res
    }
}
